//-----------------------------------------
// LScript Modeler template
//
// Copyright 2010 Michael Green
// mike@mikegreen.name
//

myversion=0.2;
@version 2.2
@warnings
@script modeler

// global values go here
glimit=1;
gsurfa=2;
gcolla=3;
gset;
gpoints;
main
{
    // requester code goes here
    gset[glimit]=recall(glimit.asStr(),20);
    gset[gsurfa]=recall(gsurfa.asStr(),"Break");
    gset[gcolla]=recall(gcolla.asStr(),1);

    reqbegin("MG Voronoi Shatter v"+myversion);

    c[glimit] = ctlinteger("Maximum ",gset[glimit]);
    c[gsurfa] = ctlstring("Surface",gset[gsurfa]);
    c[gcolla] = ctlcheckbox("Collapse",gset[gcolla]);
    t1=ctltext("Note:", "High maximum values (40 or so?)",
    			"will take longer times and increase the",
    			"likelyhood there will be errors with",
    			"Modeler's boolean function");

    return if !reqpost();

    // get requester control values here
    for(i=1;i<=gset.size();i++)
	{
		gset[i]=getvalue(c[i]);
		store(i.asStr(),gset[i]);
	}
    reqend();

    // processing code goes here
    VoronoiShatter();

}

VoronoiShatter
{
	selmode(USER);
	selpolygon(CLEAR);
	selpolygon(SET,NVGT,2);
	if(polycount()==0)
		error("Please select a mesh");
	undogroupbegin();
	copy();
	new();
	paste();
	freezecurves();
//	triple();
	MakePoints();
	CopyModel();
	MakeBlock();
	Shatter();
	if(gset[gcolla]) FlattenLayers();
	mylayers=lyrdata();
	lyrsetfg(mylayers);

	undogroupend();
}

MakePoints
{
	mylayers=lyrfg();
	mybounding=boundingbox();
	lyrsetfg(gset[glimit]+2);
	editbegin();
	for(i=1;i<=gset[glimit];i++)
	{
		myx=randu()*(mybounding[2].x-mybounding[1].x)+mybounding[1].x;
		myy=randu()*(mybounding[2].y-mybounding[1].y)+mybounding[1].y;
		myz=randu()*(mybounding[2].z-mybounding[1].z)+mybounding[1].z;
		mypoint[1]=addpoint(myx,myy,myz);
		mypoly=addpolygon(mypoint);
	}
	editend();
	lyrsetbg(mylayers[1]);
	boolean(INTERSECT);

	lyrsetfg(mylayers[1]);
	lyrsetfg(gset[glimit]+2);

	editbegin();
	i=1;
	foreach(p,points)
	{
		gpoints[i]=pointinfo(p);
		i++;
	}
	editend();
	delete();
}

CopyModel
{
	lyrsetfg(1);
	copy();
	for(i=2;i<=gpoints.size();i++)
	{
		lyrsetfg(i);
		paste();
	}
}
MakeBlock
{
	mybbox=boundingbox();
	myempty=lyrempty();
	lyrsetfg(myempty[1]);
	mymag=vmag(mybbox[2]-mybbox[1]);
	makebox(<0 - mymag,0,0 - mymag>,<mymag,mymag*2,mymag>,1);
	changesurface(gset[gsurfa]);
	scale(2);
//	triple();
}
Shatter
{
	copy();
	delete();
	temp=lyrempty();
	blocklayer=temp[1];
	moninit(gpoints.size());
	for(i=1;i<=gpoints.size();i++)
	{
		for(j=i;j<=gpoints.size();j++)
		{
        	if(i==j)
        		continue;
			paste();
        	pointmid=(gpoints[i]+gpoints[j])/2;
        	direction=gpoints[j]-gpoints[i];
			(roth,rotp)=VecToHPB(direction);
			rotate(rotp,X);
			rotate(roth,Y);
			move(pointmid);
			lyrsetfg(i);
			lyrsetbg(blocklayer);
			boolean(SUBTRACT);
			CleanUp();
			lyrsetfg(j);
			lyrsetbg(blocklayer);
			boolean(INTERSECT);
			CleanUp();
			lyrsetfg(blocklayer);
			delete();
		}
		lyrsetfg(i);
		changepart("Block_"+i.asStr());
		setlayername("Block_"+i.asStr());
		bbox=boundingbox();
		mycentre=(bbox[1]+bbox[2])/2;
		setpivot(mycentre);
		monstep();
		lyrsetfg(blocklayer);
	}
	monend();


}

CleanUp
{
	mergepoints();
//	triple();
	selmode(USER);
  	selpolygon(SET, NVLT, 3);
  	selmode(DIRECT);
  	if(polycount()>0)
  		delete();
  	selmode(USER);
  	selpoint(SET, NPLT, 1);
  	selmode(DIRECT);
  	if(pointcount()>0)
  		delete();
  	selmode(USER);
}

FlattenLayers
{
	mylayers=lyrdata();
	lyrsetfg(mylayers);
	cut();
	lyrsetfg(1);
	paste();
}

VecToHPB:pvec
{
	heading=0;
	pitch=0;
	if(pvec.z!=0)
		heading= deg(atan(pvec.x/pvec.z));
	else
	{
		if(pvec.x>0)
			heading=90;
		else if(pvec.x<0)
			heading=-90;
	}
	if(pvec.z<0)
		heading = 180 + heading;

	heading=0-heading;   // rotation is reversed in modeler


	outval=pvec.x^^2+pvec.z^^2;
	if(pvec.y==0)
		pitch=90;
	else if(outval!=0)
		pitch=deg(atan(sqrt(outval)/pvec.y));
	else if(pvec.y<0)
		pitch=180;

	if(pitch<0)
		pitch=180+pitch;

	pitch=0-pitch;       // rotation is reversed in modeler

	return(heading,pitch);
}
